SystemOrganization addCategory: #'Math-Core'!

Array variableSubclass: #DhbVector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Core'!

!DhbVector commentStamp: 'StephaneDucasse 3/2/2016 09:04' prior: 0!
A vector is an object in a multidimensional space. It is represented by its components measured on a reference system.

Here is a typical use of myself

[[[ 
| u v w a b c |
u := #(1 2 3) asVector.
v := #(3 4 5) asVector.
a := DhbMatrix rows: #( ( 1 0 1 ) (-1 -2 3)).
b := DhbMatrix rows: #( ( 1 2 3 ) (-2 1 7) (5 6 7)).
w := 4 * u + (3 * v).
c := a * b.
v := a * u.
w := c transpose * v.
w := v * c.
]]]

(c) Copyrights Didier BESSET, 1999.!

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:28'!
* aNumberOrMatrixOrVector
		"Answers the product of the receiver with the argument. The argument can be a number, matrix or vector. "
	^ aNumberOrMatrixOrVector productWithVector: self! !

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:28'!
+ aVector
	"Answers the sum of the receiver with aVector."
	| answer n |
	answer := self class new: self size.
	n := 0.
	self with: aVector do:
		[ :a :b | 
		  n := n + 1. 
		  answer at: n put: ( a + b).
		].
	^answer! !

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:27'!
- aVector
	"Answers the difference of the receiver with aVector."
	| answer n |
	answer := self class new: self size.
	n := 0.
	self with: aVector do:
		[ :a :b | 
		  n := n + 1. 
		  answer at: n put: ( a - b).
		].
	^answer! !

!DhbVector methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:35'!
accumulate: aVectorOrAnArray
	"Modify the receiver adding the contents of the argument to the receiver."
	1 to: self size do: [ :n | self at: n put: ((self at: n) + (aVectorOrAnArray at: n))].! !

!DhbVector methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:36'!
accumulateNegated: aVectorOrAnArray
	"Modify the receiver adding the negated contents of the argument to the receiver."
	1 to: self size do: [ :n | self at: n put: ((self at: n) - (aVectorOrAnArray at: n))].! !

!DhbVector methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 3/2/2016 11:47'!
asArray
	
	| array i|
	array := Array new: self size.
	i := 0.
	self do: [:item | array basicAt: (i:=i+1) put: item].
	^ array! !

!DhbVector methodsFor: 'creation' stamp: 'StephaneDucasse 7/5/2015 19:33'!
asDhbVector
	"Answer self since the receiver is a vector."
	^ self! !

!DhbVector methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:35'!
negate
	"Inverse the sign of all components of the receiver."
	1 to: self size do: [ :n | self at: n put: (self at: n) negated].! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
negated
	^ self * -1! !

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:32'!
norm
	"Answer the norm of the receiver."
	^(self * self) sqrt! !

!DhbVector methodsFor: 'creation' stamp: 'StephaneDucasse 7/5/2015 19:32'!
normalized
	^ (1 / self norm) * self! !

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:28'!
productWithMatrix: aMatrix
	"Answers the product of aMatrix with the receiver."
	^aMatrix rowsCollect: [ :each | each * self]! !

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:28'!
productWithVector: aVector
	"Answers the scalar product of aVector with the receiver."
	| n |
	n := 0.
	^self inject: 0
			into: [ :sum :each | n := n + 1. (aVector at: n) * each + sum]! !

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:28'!
scalarProduct: aVector
	
	| product n |
	n := 0.
	product := self collect: [ :each | n := n + 1. (aVector at: n) * each].
	n := product size.
	[ n > 1]
		whileTrue:[ | i j|
					i := 1.
					j := n.
					[ i < j]
						whileTrue: [ product at: i put: ( product at: i) + ( product at: j).
									 j := j - 1.
									 i := i + 1.
								   ].
					n := i min: j.
				  ].
	^product at: 1! !

!DhbVector methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:35'!
scaleBy: aNumber
	"Modify the receiver elements by a multiplicating factor."
	1 to: self size do: [ :n | self at: n put: ((self at: n) * aNumber)].! !

!DhbVector methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:29'!
tensorProduct: aVector
	"Answers the tensor product of the receiver with aVector."
	^self == aVector "dont use #="
			ifTrue: [DhbSymmetricMatrix new: self size function:[:x :y|x*y]] 
			ifFalse:[DhbMatrix rows: ( self collect: [ :a | aVector collect: [ :b | a * b]])]
! !

Object subclass: #DhbFloatingPointMachine
	instanceVariableNames: 'defaultNumericalPrecision radix machinePrecision negativeMachinePrecision smallestNumber largestNumber smallNumber largestExponentArgument'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-Core'!

!DhbFloatingPointMachine commentStamp: 'StephaneDucasse 3/2/2016 07:39' prior: 0!
A DhbFloatingPointMachine represents the numerical precision of this system.

Instance Variables

	defaultNumericalPrecision:		The relative 
numerical precision that can be expected for a general numerical computation. One should consider to numbers a and b equal if the relative difference between them is less than the default machine precision.

	largestExponentArgument:		natural logarithm of largest number

	largestNumber:		The largest positive number that can be represented in the machine

	machinePrecision:		r^{-(n+1)}, with the largest n such that (1 + r^-n) - 1 !!= 0

	negativeMachinePrecision:		r^{-(n+1)}, with the largest n such that (1 - r^-n) - 1 !!= 0

	radix:		The radix of the floating point representation. This is often 2.

	smallNumber:		A number that can be added to some value without noticeably changing the result of the computation

	smallestNumber:		The smallest positive number different from 0.


largestExponentArgument
	- xxxxx

This class is detailed in Object Oriented Implementation of Numerical Methods, Section 1.4.1 and 1.4.2.

(c) Copyrights Didier BESSET, 1999.
!

!DhbFloatingPointMachine class methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 22:05'!
new
	
	UniqueInstance ifNil: [ UniqueInstance := super new].
	^ UniqueInstance! !

!DhbFloatingPointMachine class methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 22:05'!
reset

	UniqueInstance := nil.! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
computeLargestNumber

	| one floatingRadix fullMantissaNumber |
	one := 1.0.
	floatingRadix := self radix asFloat.
	fullMantissaNumber := one - ( floatingRadix * self negativeMachinePrecision).
	largestNumber := fullMantissaNumber.
	[ [ fullMantissaNumber := fullMantissaNumber * floatingRadix.
	    fullMantissaNumber isInfinite ifTrue: [^nil].
	    largestNumber := fullMantissaNumber.
		true] whileTrue: [ ].
		] on: Error do: [ :signal | signal return: nil].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
computeMachinePrecision
	
	| one zero inverseRadix tmp |
	one := 1.0.
	zero := 0.0.
	inverseRadix := one / self radix asFloat.
	machinePrecision := one.
	[ tmp := one + machinePrecision.
	  tmp - one = zero]
		whileFalse:[  machinePrecision := machinePrecision * inverseRadix].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
computeNegativeMachinePrecision
	
	| one zero floatingRadix inverseRadix tmp |
	one := 1.0.
	zero := 0.0.
	floatingRadix := self radix asFloat.
	inverseRadix := one / floatingRadix.
	negativeMachinePrecision := one.
	[ tmp := one - negativeMachinePrecision.
	  tmp - one = zero]
		whileFalse:[ negativeMachinePrecision := negativeMachinePrecision * inverseRadix].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
computeRadix
	
	| one zero a b tmp1 tmp2|
	one := 1.0.
	zero := 0.0.
	a := one.
	[ a := a + a.
	  tmp1 := a + one.
	  tmp2 := tmp1 - a.
	  tmp2 - one = zero] whileTrue:[].
	b := one.
	[ b := b + b.
	  tmp1 := a + b.
	  radix := ( tmp1 - a) truncated.
	  radix = 0 ] whileTrue: [].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
computeSmallestNumber
	
	| one floatingRadix inverseRadix fullMantissaNumber |
	one := 1 asFloat.
	floatingRadix := self radix asFloat.
	inverseRadix := one / floatingRadix.
	fullMantissaNumber := one - (floatingRadix * self negativeMachinePrecision).
	smallestNumber := fullMantissaNumber.
	[[fullMantissaNumber := fullMantissaNumber * inverseRadix.
	fullMantissaNumber = 0.0 ifTrue: [Error signal ].
	smallestNumber := fullMantissaNumber.
	true]
		whileTrue: []]
		on: Error do: [:signal | signal return: nil]! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
defaultNumericalPrecision

	defaultNumericalPrecision isNil
		ifTrue: [ defaultNumericalPrecision := self machinePrecision sqrt].
	^defaultNumericalPrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
largestExponentArgument
	
	largestExponentArgument isNil
		ifTrue: [ largestExponentArgument := self largestNumber ln].
	^largestExponentArgument! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
largestNumber
	
	largestNumber isNil
		ifTrue: [ self computeLargestNumber].
	^largestNumber! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:33'!
machinePrecision
	
	machinePrecision isNil
		ifTrue: [ self computeMachinePrecision].
	^machinePrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:34'!
negativeMachinePrecision
	
	negativeMachinePrecision isNil
		ifTrue: [ self computeNegativeMachinePrecision].
	^negativeMachinePrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:34'!
radix
	
	radix isNil
		ifTrue: [ self computeRadix].
	^radix! !

!DhbFloatingPointMachine methodsFor: 'display' stamp: 'StephaneDucasse 3/1/2016 21:33'!
showParameters

		"added 'Transcript flush' to make sure that the transcript window gets updated"
	Transcript cr; cr;
			nextPutAll: 'Floating-point machine parameters'; cr;
			nextPutAll: '---------------------------------';cr;
			nextPutAll: 'Radix: '.
	self radix printOn: Transcript.
	Transcript cr; nextPutAll: 'Machine precision: '.
	self machinePrecision printOn: Transcript.
	Transcript cr; nextPutAll: 'Negative machine precision: '.
	self negativeMachinePrecision printOn: Transcript.
	Transcript cr; nextPutAll: 'Smallest number: '.
	self smallestNumber printOn: Transcript.
	Transcript cr; nextPutAll: 'Largest number: '.
	self largestNumber printOn: Transcript.
	Transcript flush.! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:34'!
smallNumber
	
	smallNumber isNil
		ifTrue: [ smallNumber := self smallestNumber sqrt].
	^smallNumber! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:34'!
smallestNumber
	
	smallestNumber isNil
		ifTrue: [ self computeSmallestNumber].
	^smallestNumber! !

Object subclass: #DhbWeightedPoint
	instanceVariableNames: 'xValue yValue weight error'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Core'!

!DhbWeightedPoint commentStamp: 'StephaneDucasse 3/2/2016 09:04' prior: 0!
I'm a simple point (two values with a weight and an error).

(c) Copyrights Didier BESSET, 1999.!

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'StephaneDucasse 7/12/2015 10:48'!
point: aPoint
	
	^ self new initialize: aPoint weight: 1! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 22:05'!
point: aNumber count: anInteger
	
	^ self
		point: aNumber @ anInteger
		weight:
			(anInteger > 0
				ifTrue: [ 1 / anInteger ]
				ifFalse: [ 1 ])! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'StephaneDucasse 7/12/2015 10:48'!
point: aPoint error: aNumber
	
	^ self new initialize: aPoint error: aNumber! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'StephaneDucasse 7/12/2015 10:48'!
point: aPoint weight: aNumber
	
	^ self basicNew initialize: aPoint weight: aNumber! !

!DhbWeightedPoint methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:47'!
chi2ComparisonContribution: aWeightedPoint

	^ (aWeightedPoint yValue - yValue) squared / (1 / aWeightedPoint weight + (1 / weight))! !

!DhbWeightedPoint methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:47'!
chi2Contribution: aFunction

	^ (yValue - (aFunction value: xValue)) squared * weight! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'StephaneDucasse 7/12/2015 10:47'!
error

	error isNil
		ifTrue: [ error := 1 / weight sqrt ].
	^ error! !

!DhbWeightedPoint methodsFor: 'initialization' stamp: 'StephaneDucasse 7/12/2015 10:47'!
initialize: aPoint error: aNumber

	error := aNumber.
	^ self initialize: aPoint weight: 1 / aNumber squared! !

!DhbWeightedPoint methodsFor: 'initialization' stamp: 'StephaneDucasse 7/12/2015 10:47'!
initialize: aPoint weight: aNumber

	xValue := aPoint x.
	yValue := aPoint y.
	weight := aNumber.
	^ self! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'StephaneDucasse 7/12/2015 10:47'!
point

	^ xValue @ yValue! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
weight
	^weight! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
xValue
	^xValue! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
yValue
	^yValue! !
